@@ -276,18 +276,22 @@ def instantiate_node_groups(
                 node._replace(path=[], name=os.path.basename(output))  # type: ignore [call-arg] # noqa: E501
                 for node in node_group.nodes
             ]
-        instantiated_nodes = listing.instantiate_nodes(
+        copy_dir_contents = always_copy_dir_contents or source_path.endswith("/")
+        instantiated_nodes = listing.collect_nodes_to_instantiate(
             node_group.nodes,
-            output_dir,
-            cache,
-            total_files,
-            recursive=recursive,
-            copy_dir_contents=always_copy_dir_contents or source_path.endswith("/"),
-            force=force,
-            virtual_only=virtual_only,
-            relative_path=source_path,
-            shared_progress_bar=instantiate_progress_bar,
+            recursive,
+            copy_dir_contents,
+            source_path,
         )
+        if not virtual_only:
+            listing.instantiate_nodes(
+                instantiated_nodes,
+                output_dir,
+                cache,
+                total_files,
+                force=force,
+                shared_progress_bar=instantiate_progress_bar,
+            )
 
         for node in instantiated_nodes:
             if not node.is_dir:
@@ -336,14 +340,14 @@ class Catalog:
             tmp_dir=dql_dir.tmp,
         )
 
-    def add_storage(self, source: str) -> None:
+    def add_storage(self, source: str, symlinks: bool = False) -> None:
         # pylint:disable-next=protected-access
         path = LocalFileSystem._strip_protocol(source)
         if not os.path.isdir(path):
             raise RuntimeError(f"{path} is not a directory")
         uri = Path(path).as_uri()
         print(f"Registering storage {uri}")
-        self.data_storage.create_storage_if_not_registered(uri)
+        self.data_storage.create_storage_if_not_registered(uri, symlinks)
 
     def parse_url(self, uri: str, **config: Any) -> Tuple[Client, str]:
         return Client.parse_url(uri, self.data_storage, **config)
@@ -403,6 +407,7 @@ class Catalog:
             try:
                 source_data_storage.init_db(prefix, is_new)
                 lst.fetch(prefix)
+                source_data_storage.validate_paths()
                 source_data_storage.mark_storage_indexed(
                     storage.uri,
                     Status.PARTIAL if prefix else Status.COMPLETE,
@@ -521,37 +526,79 @@ class Catalog:
                 name, client.uri, path, recursive=recursive
             )
 
-    def register(
+    def register_shadow_dataset(
         self,
-        dataset_name: str,
-        new_dataset_name: Optional[str] = None,
+        shadow_name: str,
+        registered_name: Optional[str] = None,
         version: Optional[int] = None,
         description: Optional[str] = None,
         labels: Optional[List[str]] = None,
     ):
+        """
+        Method for registering shadow dataset as a new dataset with version 1, or
+        as a new version of existing registered dataset
+        """
         version = version or 1
 
-        dataset = self.data_storage.get_dataset(dataset_name)
-        if not dataset:
-            raise ValueError(f"Dataset {dataset_name} doesn't exist")
+        # getting shadow dataset
+        shadow_dataset = self.data_storage.get_dataset(shadow_name)
+        if not shadow_dataset:
+            raise ValueError(f"Shadow dataset {shadow_name} doesn't exist")
 
-        if not dataset.shadow:
-            raise ValueError(f"Dataset {dataset_name} is already registered")
+        if not shadow_dataset.shadow:
+            raise ValueError(f"Dataset {shadow_name} must be shadow")
 
+        registered_dataset = (
+            self.data_storage.get_dataset(registered_name) if registered_name else None
+        )
+
+        if registered_dataset:
+            # if registered dataset already exists, we are creating new version
+            # of it out of shadow dataset
+            version = version or registered_dataset.next_version
+            if not registered_dataset.is_valid_next_version(version):
+                raise ValueError(
+                    f"Version {version} must be higher than the current latest one"
+                )
+
+            self.data_storage.create_dataset_version(
+                registered_dataset.name, version, create_rows_table=False
+            )
+            # to avoid re-creating rows table, we are just taking shadow one and
+            # renaming it for a new version of registered one
+            self.data_storage.rename_dataset_table(
+                shadow_dataset.name,
+                registered_dataset.name,
+                old_version=None,
+                new_version=version,
+            )
+            # finally, we are removing shadow dataset from datasets table
+            self.data_storage.remove_shadow_dataset(shadow_dataset, drop_rows=False)
+            return
+
+        # if registered dataset doesn't exist we are modifying shadow dataset
+        # to become registered one
         update_data = {
             "shadow": False,
             "description": description,
             "labels": labels,
         }
 
-        if new_dataset_name:
-            update_data["name"] = new_dataset_name
+        if registered_name:
+            update_data["name"] = registered_name
 
-        self.data_storage.update_dataset(dataset_name, **update_data)
+        self.data_storage.update_dataset(shadow_name, **update_data)
         self.data_storage.create_dataset_version(
-            new_dataset_name or dataset_name, version, create_rows_table=False
+            registered_name or shadow_name,
+            version,
+            create_rows_table=False,
+        )
+        self.data_storage.rename_dataset_table(
+            registered_name or shadow_name,
+            registered_name or shadow_name,
+            old_version=None,
+            new_version=version,
         )
-        self.data_storage.rename_dataset_rows(new_dataset_name or dataset_name, version)
 
     def ls_datasets(self, shadow_only=None) -> Iterable[DatasetRecord]:
         yield from self.data_storage.get_datasets(shadow_only=shadow_only)
