@@ -1,12 +1,17 @@
 import asyncio
 import multiprocessing
+import os
 import posixpath
 from abc import abstractmethod
 from datetime import datetime
+from shutil import copy2
 from typing import TYPE_CHECKING, ClassVar, Iterator, Tuple, Type
 
 from botocore.exceptions import ClientError
+from dvc_data.hashfile.db.local import LocalHashFileDB
 from fsspec.asyn import get_loop
+from reflink import reflink
+from reflink.error import ReflinkImpossibleError
 from tqdm import tqdm
 
 from dql.client.base import Bucket, Client
@@ -17,6 +22,8 @@ from dql.nodes_thread_pool import NodeChunk
 if TYPE_CHECKING:
     from fsspec.spec import AbstractFileSystem
 
+    from dql.node import NodeWithPath
+
 FETCH_WORKERS = 100
 
 
@@ -37,7 +44,7 @@ class FSSpecClient(Client):
         return fs
 
     @classmethod
-    def from_url(cls, url: str, **kwargs):
+    def from_url(cls, url: str, data_storage, kwargs):
         return cls(url, cls.create_fs(**kwargs))
 
     @classmethod
@@ -198,3 +205,32 @@ class FSSpecClient(Client):
         if len(target_name) > max_len:
             target_name = "..." + target_name[max_len - 3 :]
         return target_name
+
+    def instantiate_node(
+        self,
+        node: "NodeWithPath",
+        cache: LocalHashFileDB,
+        output: str,
+        progress_bar: tqdm,
+        force: bool = False,
+    ) -> None:
+        if not node.name:
+            return
+        dst = os.path.join(output, *node.path)  # type: ignore[attr-defined]
+        if os.path.exists(dst):
+            if force:
+                os.remove(dst)
+            else:
+                progress_bar.close()
+                raise FileExistsError(f"Path {dst} already exists")
+        self.do_instantiate_node(node, cache, dst)
+
+    def do_instantiate_node(
+        self, node: "NodeWithPath", cache: LocalHashFileDB, dst: str
+    ) -> None:
+        src = cache.oid_to_path(node.checksum)  # type: ignore[attr-defined]
+        try:
+            reflink(src, dst)
+        except (NotImplementedError, ReflinkImpossibleError):
+            # Default to copy if reflinks are not supported
+            copy2(src, dst)
