@@ -45,7 +45,13 @@ class AbstractDataStorage(ABC):
         """Inserts data to shadow dataset based on bucket uri and glob path"""
 
     @abstractmethod
-    def rename_dataset_rows(self, name: str, version: int) -> None:
+    def rename_dataset_table(
+        self,
+        old_name: str,
+        new_name: str,
+        old_version: Optional[int] = None,
+        new_version: Optional[int] = None,
+    ) -> None:
         """
         When registering dataset, we need to rename rows table from
         ds_<id>_shadow to ds_<id>_<version>.
@@ -95,9 +101,9 @@ class AbstractDataStorage(ABC):
         """
 
     @abstractmethod
-    def remove_shadow_dataset(self, dataset: "DatasetRecord") -> None:
+    def remove_shadow_dataset(self, dataset: "DatasetRecord", drop_rows=True) -> None:
         """
-        Removes shadow dataset and it's corresponding rows
+        Removes shadow dataset and it's corresponding rows if needed
         """
 
     @abstractmethod
@@ -126,7 +132,9 @@ class AbstractDataStorage(ABC):
         """Gets nodes from database by parent_id, with optional filtering"""
 
     @abstractmethod
-    def create_storage_if_not_registered(self, uri: str) -> None:
+    def create_storage_if_not_registered(
+        self, uri: str, symlinks: bool = False
+    ) -> None:
         """
         Saves new storage if it doesn't exist in database
         """
@@ -173,6 +181,12 @@ class AbstractDataStorage(ABC):
         This method should be called when index operation is finished
         """
 
+    @abstractmethod
+    def validate_paths(self) -> int:
+        """
+        Find and mark any invalid paths.
+        """
+
     @abstractmethod
     def get_storage_all(self) -> Iterator[Storage]:
         pass
