@@ -10,6 +10,9 @@ from itertools import groupby
 from time import sleep
 from typing import Any, Dict, Iterable, Iterator, List, Optional, Tuple, Union
 
+import sqlalchemy
+from sqlalchemy import select, update
+
 from dql.data_storage.abstract import AbstractDataStorage
 from dql.dataset import DatasetRecord, DatasetRow
 from dql.error import DQLError
@@ -26,6 +29,13 @@ RETRY_START_SEC = 0.01
 RETRY_MAX_TIMES = 10
 RETRY_FACTOR = 2
 
+# sqlite 3.31.1 is the earliest version tested in CI
+if sqlite3.sqlite_version_info < (3, 31, 1):
+    logger.warning(
+        "Possible sqlite incompatibility. The earliest tested version of "
+        f"sqlite is 3.31.1 but you have {sqlite3.sqlite_version}"
+    )
+
 
 def adapt_datetime(val: datetime) -> str:
     if not (val.tzinfo is timezone.utc or val.tzname() == "UTC"):
@@ -65,6 +75,7 @@ class NodeSchema(Schema):
         "owner_id",
         "path_str",
         "anno",
+        "valid",
     ]
 
     def lookup_type(self, query, _, value):
@@ -114,6 +125,7 @@ class StorageSchema(Schema):
         "started_inserting_at",
         "last_inserted_at",
         "status",
+        "symlinks",
     ]
 
 
@@ -170,9 +182,10 @@ class SQLiteDataStorage(AbstractDataStorage):
     TABLE_NAME_SHA_LIMIT = 12
 
     def __init__(self, db_file: Optional[str] = None, uri: str = ""):
-        self.uri = uri
+        self.uri = uri.rstrip("/")
         self.db_file = db_file if db_file else DQLDir.find().db
         detect_types = sqlite3.PARSE_DECLTYPES | sqlite3.PARSE_COLNAMES
+
         try:
             if self.db_file == ":memory:":
                 # Enable multithreaded usage of the same in-memory db
@@ -181,6 +194,10 @@ class SQLiteDataStorage(AbstractDataStorage):
                 )
             else:
                 self.db = sqlite3.connect(self.db_file, detect_types=detect_types)
+            self.engine = sqlalchemy.create_engine(
+                "sqlite+pysqlite:///", creator=lambda: self.db
+            )
+
             self.db.isolation_level = None  # Use autocommit mode
             self.db.execute("PRAGMA foreign_keys = ON")
             self.db.execute("PRAGMA cache_size = -102400")  # 100 MiB
@@ -193,6 +210,18 @@ class SQLiteDataStorage(AbstractDataStorage):
             self._init_datasets_tables()
         except RuntimeError:
             raise DQLError("Can't connect to SQLite DB")
+        self._table = None
+
+    @property
+    def table(self):
+        table_name = self.table_name
+        if table_name is None:
+            return None
+        elif self._table is None or self._table.name != table_name:
+            self._table = sqlalchemy.table(
+                table_name, *[sqlalchemy.column(f) for f in NodeSchema.fields]
+            )
+        return self._table
 
     def _init_storage_table(self):
         """Initialize only tables related to storage, e.g s3"""
@@ -205,7 +234,8 @@ class SQLiteDataStorage(AbstractDataStorage):
                 expires     DATETIME,
                 started_inserting_at  DATETIME,
                 last_inserted_at  DATETIME,
-                status      INTEGER NOT NULL
+                status      INTEGER NOT NULL,
+                symlinks    BOOL NOT NULL
             )
         """
         )
@@ -259,7 +289,8 @@ class SQLiteDataStorage(AbstractDataStorage):
                 owner_name      TEXT,
                 owner_id        TEXT,
                 path_str        TEXT,
-                anno            JSON
+                anno            JSON,
+                valid           BOOL DEFAULT 1 NOT NULL
             );
 
             CREATE INDEX IF NOT EXISTS idx_dir_type_{self.table_name}
@@ -346,7 +377,9 @@ class SQLiteDataStorage(AbstractDataStorage):
         def _with_path(row):
             return NodeWithPath(*row, path_list)  # type: ignore [call-arg]
 
-        q = self.nodes.where(parent_id=node.id, is_latest=True, name__glob=glob_name)
+        q = self.nodes.where(
+            valid=True, parent_id=node.id, is_latest=True, name__glob=glob_name
+        )
         return q.wrap(_with_path).all()
 
     def _get_node_by_path_list(self, path_list: List[str]) -> NodeWithPath:
@@ -354,15 +387,15 @@ class SQLiteDataStorage(AbstractDataStorage):
         Gets node that correspond some path list, e.g ["data-lakes", "dogs-and-cats"]
         """
         path_str = "/".join(path_list)
-        nodes = (
+        node = (
             self.nodes.with_path()
-            .where(path_str=path_str, is_latest=True)
+            .where(valid=True, path_str=path_str, is_latest=True)
             .order_by("dir_type")
-            .all()
+            .get()
         )
-        if not nodes:
+        if not node:
             raise FileNotFoundError(f"Unable to resolve path {path_str}")
-        return nodes[-1]
+        return node
 
     def _populate_nodes_by_path(
         self, path_list: List[str], num: int, res: List[NodeWithPath]
@@ -423,6 +456,7 @@ class SQLiteDataStorage(AbstractDataStorage):
             "name": "",
             "is_latest": True,
             "size": 0,
+            "valid": True,
             **d,
         }
         return {f: d.get(f) for f in NodeSchema.fields[1:]}
@@ -502,7 +536,7 @@ class SQLiteDataStorage(AbstractDataStorage):
         ).fetchall():
             yield DatasetRow(*row)
 
-    def create_shadow_dataset(self, name: str) -> None:
+    def create_shadow_dataset(self, name: str, create_rows: bool = True) -> None:
         """Creates new shadow dataset if it doesn't exist yet"""
         with self.db:
             self.db.execute("begin")
@@ -517,8 +551,9 @@ class SQLiteDataStorage(AbstractDataStorage):
             dataset = self.get_dataset(name)
             assert dataset
             assert dataset.shadow
-            table_name = self._dataset_table_name(dataset.id)
-            self.create_dataset_rows_table(table_name)
+            if create_rows:
+                table_name = self._dataset_table_name(dataset.id)
+                self.create_dataset_rows_table(table_name)
 
     def insert_into_shadow_dataset(
         self, name: str, uri: str, path: str, recursive=False
@@ -534,10 +569,10 @@ class SQLiteDataStorage(AbstractDataStorage):
         if recursive:
             if not path.endswith("*"):
                 path = path.rstrip("/") + "/*"  # glob filter must end with /*
-            nodes = self.nodes.where(path_str__glob=path, type="file")
+            nodes = self.nodes.where(valid=True, path_str__glob=path, type="file")
         else:
             parent = self.get_node_by_path(path.lstrip("/").rstrip("/*"))
-            nodes = self.nodes.where(parent_id=parent.id, type="file")
+            nodes = self.nodes.where(valid=True, parent_id=parent.id, type="file")
 
         (
             where_sql,
@@ -585,12 +620,20 @@ class SQLiteDataStorage(AbstractDataStorage):
         )
         self.db.commit()
 
-    def rename_dataset_rows(self, name: str, version: int) -> None:
-        dataset = self.get_dataset(name)
-        assert dataset
+    def rename_dataset_table(
+        self,
+        old_name: str,
+        new_name: str,
+        old_version: Optional[int] = None,
+        new_version: Optional[int] = None,
+    ) -> None:
+        old_dataset = self.get_dataset(old_name)
+        new_dataset = self.get_dataset(new_name)
+        assert old_dataset
+        assert new_dataset
 
-        old_name = self._dataset_table_name(dataset.id, None)
-        new_name = self._dataset_table_name(dataset.id, version)
+        old_name = self._dataset_table_name(old_dataset.id, old_version)
+        new_name = self._dataset_table_name(new_dataset.id, new_version)
 
         self.db.execute(f"ALTER TABLE {old_name} RENAME TO {new_name}")
 
@@ -709,9 +752,12 @@ class SQLiteDataStorage(AbstractDataStorage):
 
         self.db.execute(f"DROP TABLE {self._dataset_table_name(dataset.id, version)}")
 
-    def remove_shadow_dataset(self, dataset: DatasetRecord) -> None:
-        self.db.execute("DELETE FROM datasets WHERE id = ?", [dataset.id])
-        self.db.execute(f"DROP TABLE {self._dataset_table_name(dataset.id)}")
+    def remove_shadow_dataset(self, dataset: DatasetRecord, drop_rows=True) -> None:
+        with self.db:
+            self.db.execute("begin")
+            self.db.execute("DELETE FROM datasets WHERE id = ?", [dataset.id])
+            if drop_rows:
+                self.db.execute(f"DROP TABLE {self._dataset_table_name(dataset.id)}")
 
     @retry_sqlite_locks_async
     async def update_last_inserted_at(self, uri: Optional[str] = None) -> None:
@@ -736,7 +782,7 @@ class SQLiteDataStorage(AbstractDataStorage):
         type: Optional[str] = None,  # pylint: disable=redefined-builtin
     ) -> Iterable[Node]:
         """Gets nodes from database by parent_id, with optional filtering"""
-        return self.nodes.where(parent_id=parent_id, type=type).all()
+        return self.nodes.where(valid=True, parent_id=parent_id, type=type).all()
 
     def get_storage_all(self) -> Iterator[Storage]:
         return self.storages.all()
@@ -745,8 +791,13 @@ class SQLiteDataStorage(AbstractDataStorage):
         return self.storages.where(uri=uri).get()
 
     @retry_sqlite_locks
-    def create_storage_if_not_registered(self, uri: str) -> None:
-        self.storages.insert({"uri": uri, "status": StorageStatus.CREATED}, silent=True)
+    def create_storage_if_not_registered(
+        self, uri: str, symlinks: bool = False
+    ) -> None:
+        self.storages.insert(
+            {"uri": uri, "status": StorageStatus.CREATED, "symlinks": symlinks},
+            silent=True,
+        )
 
     def find_stale_storages(self):
         """
@@ -888,13 +939,49 @@ class SQLiteDataStorage(AbstractDataStorage):
             elif status == StorageStatus.PARTIAL:
                 dirprefix = posixpath.join(prefix, "")
                 row = {"path_str": dirprefix, "timestamp": end_time, "expires": expires}
-                self.partials.insert(row)
+                try:
+                    self.partials.insert(row)
+                # If path_str unique constraint is violated, do an update.
+                # Exception type depends on db.
+                except Exception:  # pylint: disable=broad-except
+                    self.partials.where(path_str=dirprefix).update(
+                        timestamp=end_time, expires=expires
+                    )
+
+    def validate_paths(self) -> int:
+        assert isinstance(self.table_name, str), "table_name must be a string"
+        t1 = self.table.alias("t1")
+        t2 = self.table.alias("t2")
+        t3 = self.table.alias("t3")
+        id_query = (
+            select(t2.c.id)  # type: ignore[attr-defined]
+            .select_from(t2)
+            .join(t3, (t2.c.path_str == t3.c.path_str) & (t2.c.id != t3.c.id))
+            .where(
+                t2.c.valid == True,  # noqa: E712 pylint:disable=singleton-comparison
+                t2.c.dir_type == 0,
+            )
+        )
+        query = (
+            update(t1)
+            .values(valid=False)
+            .where(t1.c.id.in_(id_query))  # type: ignore[attr-defined]
+        )
+
+        with self.engine.connect() as con:
+            row_count = con.execute(query).rowcount
+        if row_count:
+            logger.warning(
+                "File names that collide with directory names will be ignored. "
+                f"Number found: {row_count}"
+            )
+        return row_count
 
     def get_node_by_path(self, path: str) -> NodeWithPath:
         conds: Dict[str, Any] = {"path_str": path.strip("/")}
         if path.endswith("/"):
             conds["dir_type__ne"] = DirType.FILE
-        node = self.nodes.with_path().where(**conds).get()
+        node = self.nodes.with_path().where(valid=True, **conds).get()
         if not node:
             raise FileNotFoundError(f"Unable to resolve path {path}")
         return node
@@ -912,7 +999,9 @@ class SQLiteDataStorage(AbstractDataStorage):
         if not parent_node.is_dir:
             return [parent_node]
 
-        return self.nodes.where(parent_id=parent_node.id, is_latest=True).all()
+        return self.nodes.where(
+            valid=True, parent_id=parent_node.id, is_latest=True
+        ).all()
 
     def size(self, node: Node) -> Tuple[int, int]:
         if not node.is_dir:
@@ -920,9 +1009,9 @@ class SQLiteDataStorage(AbstractDataStorage):
             return node.size, 1
 
         sub_glob = posixpath.join(node.path_str, "*")
-        size, count = self.nodes.where(path_str__glob=sub_glob, is_latest=True).get(
-            "SUM(size)", "SUM(dir_type = 0)"
-        )
+        size, count = self.nodes.where(
+            valid=True, path_str__glob=sub_glob, is_latest=True
+        ).get("SUM(size)", "SUM(dir_type = 0)")
         return size or 0, count or 0
 
     def walk_subtree(
@@ -946,22 +1035,15 @@ class SQLiteDataStorage(AbstractDataStorage):
         if type is not None:
             conds["type"] = type
 
-        q = self.nodes.with_path(cut=node.path_str).where(**conds)
+        q = self.nodes.with_path(cut=node.path_str).where(valid=True, **conds)
         if sort:
             q = q.order_by(sort)
-        results = q.all()
-
-        if not results:
-            raise FileNotFoundError(f"Unable to resolve path {node.path_str}")
-        return results
+        return q.all()
 
     def find(self, node: AnyNode, jmespath="", **conds) -> Iterable[NodeWithPath]:
         if jmespath:
             raise NotImplementedError("jmespath queries not supported!")
-        try:
-            return self.walk_subtree(node, _conds=conds)
-        except FileNotFoundError:
-            return []
+        return self.walk_subtree(node, _conds=conds)
 
     @retry_sqlite_locks
     def update_annotation(self, node: Node, annotation_content: str) -> None:
@@ -970,7 +1052,11 @@ class SQLiteDataStorage(AbstractDataStorage):
         names = [node.name_no_ext + "." + ext for ext in img_exts]
 
         res = self.nodes.where(
-            parent_id=node.parent_id, type="file", is_latest=True, name__ieq=names
+            valid=True,
+            parent_id=node.parent_id,
+            type="file",
+            is_latest=True,
+            name__ieq=names,
         )
 
         if res is None or len(res) == 0:
