@@ -1,15 +1,11 @@
 import os
 from collections import defaultdict
 from itertools import zip_longest
-from shutil import copy2
 from typing import DefaultDict, List
 
 import yaml
-from dvc_data.hashfile.db.local import LocalHashFileDB
 from fsspec.asyn import get_loop, sync
 from funcy import notnone, select_values
-from reflink import reflink
-from reflink.error import ReflinkImpossibleError
 from tqdm import tqdm
 
 from dql.client import Client
@@ -26,30 +22,6 @@ def check_checksums(nodes):
             raise ValueError(f"Instantiation Error: Missing checksum for node: {node}")
 
 
-def instantiate_node(
-    node: NodeWithPath,
-    cache: LocalHashFileDB,
-    output: str,
-    progress_bar: tqdm,
-    force: bool = False,
-):
-    if not node.name:
-        return
-    src = cache.oid_to_path(node.checksum)  # type: ignore[attr-defined]
-    dst = os.path.join(output, *node.path)  # type: ignore[attr-defined]
-    if os.path.exists(dst):
-        if force:
-            os.remove(dst)
-        else:
-            progress_bar.close()
-            raise FileExistsError(f"Path {dst} already exists")
-    try:
-        reflink(src, dst)
-    except (NotImplementedError, ReflinkImpossibleError):
-        # Default to copy if reflinks are not supported
-        copy2(src, dst)
-
-
 class Listing:
     def __init__(
         self,
@@ -147,18 +119,12 @@ class Listing:
                 if not node.is_dir:
                     node.append_to_file(fd)
 
-    def instantiate_nodes(
+    def collect_nodes_to_instantiate(
         self,
         nodes,
-        output,
-        cache,
-        total_files=None,
         recursive=False,
         copy_dir_contents=False,
-        force=False,
-        virtual_only=False,
         relative_path=None,
-        shared_progress_bar=None,
     ):
         rel_path_elements = relative_path.split("/") if relative_path else []
         all_nodes = []
@@ -183,12 +149,18 @@ class Listing:
                 )
             else:
                 all_nodes.append(node._replace(path=node_path + [node.name]))
+        return all_nodes
 
-        if virtual_only:
-            return all_nodes
-
+    def instantiate_nodes(
+        self,
+        all_nodes,
+        output,
+        cache,
+        total_files=None,
+        force=False,
+        shared_progress_bar=None,
+    ):
         check_checksums(all_nodes)
-
         nodes = iter(all_nodes)
 
         progress_bar = shared_progress_bar or tqdm(
@@ -202,7 +174,6 @@ class Listing:
         os.makedirs(output, exist_ok=True)
         node = next(nodes, None)
         counter = 0
-
         while node:
             curr_dir_path = node.path[:-1]
             curr_dir_path_str = os.path.join(output, *curr_dir_path)
@@ -211,28 +182,20 @@ class Listing:
             os.makedirs(curr_dir_path_str, exist_ok=True)
 
             while node and node.parent_id == curr_parent_id:
-                instantiate_node(node, cache, output, progress_bar, force)
-
+                self.client.instantiate_node(node, cache, output, progress_bar, force)
                 counter += 1
                 if counter > 1000:
                     progress_bar.update(counter)
                     counter = 0
-
                 node = next(nodes, None)
 
         progress_bar.update(counter)
 
-        return all_nodes
-
     def instantiate_subtree(self, node, output, cache, total_files=None):
-        self.instantiate_nodes(
-            [node],
-            output,
-            cache,
-            total_files,
-            recursive=True,
-            copy_dir_contents=True,
+        nodes = self.collect_nodes_to_instantiate(
+            [node], recursive=True, copy_dir_contents=True
         )
+        self.instantiate_nodes(nodes, output, cache, total_files)
 
     def index_open_image_format(
         self,
