@@ -1,9 +1,15 @@
 import os
 import os.path as osp
 import stat
-from typing import Iterable, Optional, Type, TypeVar
+from datetime import datetime
+from typing import TYPE_CHECKING, Iterable, Mapping, Optional, Type, TypeVar, Union
 
+from dateutil import tz
 from dateutil.parser import isoparse
+from tomlkit import parse
+
+if TYPE_CHECKING:
+    from tomlkit import TOMLDocument
 
 GLOB_CHARS = ["?", "*", "[", "]"]
 
@@ -24,22 +30,14 @@ class DQLDir:
         tmp: Optional[str] = None,
         db: Optional[str] = None,
     ) -> None:
-        self.root = (
-            osp.abspath(root) if root is not None else self.default_root()
-        )
+        self.root = osp.abspath(root) if root is not None else self.default_root()
         self.cache = (
-            osp.abspath(cache)
-            if cache is not None
-            else osp.join(self.root, self.CACHE)
+            osp.abspath(cache) if cache is not None else osp.join(self.root, self.CACHE)
         )
         self.tmp = (
-            osp.abspath(tmp)
-            if tmp is not None
-            else osp.join(self.root, self.TMP)
-        )
-        self.db = (
-            osp.abspath(db) if db is not None else osp.join(self.root, self.DB)
+            osp.abspath(tmp) if tmp is not None else osp.join(self.root, self.TMP)
         )
+        self.db = osp.abspath(db) if db is not None else osp.join(self.root, self.DB)
 
     def init(self):
         os.makedirs(self.root, exist_ok=True)
@@ -66,6 +64,58 @@ class DQLDir:
         return instance
 
 
+def read_config(dql_root: str) -> Optional["TOMLDocument"]:
+    project_path = osp.join(dql_root, "config")
+    if osp.isfile(project_path):
+        with open(project_path, encoding="utf-8") as f:
+            text = f.read()
+        return parse(text)
+    return None
+
+
+def get_remote_config(
+    config: Optional["TOMLDocument"], remote: str = ""
+) -> Mapping[str, str]:
+    if config is None:
+        return {"type": "local"}
+    if not remote:
+        try:
+            remote = config["core"]["default-remote"]  # type: ignore[index,assignment] # noqa: E501
+        except KeyError:
+            return {"type": "local"}
+    try:
+        remote_conf: Mapping[str, str] = config["remote"][remote]  # type: ignore[assignment,index] # noqa: E501
+    except KeyError:
+        raise Exception(f"missing config section for default remote: remote.{remote}")
+    except Exception as exc:
+        raise Exception("invalid config") from exc
+
+    if not isinstance(remote_conf, Mapping):
+        raise Exception(f"config section remote.{remote} must be a mapping")
+
+    remote_type = remote_conf.get("type")
+    if remote_type not in ("local", "http"):
+        raise Exception(
+            f'config section remote.{remote} must have "type" with one of: '
+            '"local", "http"'
+        )
+
+    if remote_type == "http":
+        for key in ["url", "username", "token"]:
+            try:
+                remote_conf[key]
+            except KeyError:
+                raise Exception(
+                    f"config section remote.{remote} of type {remote_type} "
+                    f"must contain key {key}"
+                )
+    elif remote_type != "local":
+        raise Exception(
+            f"config section remote.{remote} has invalid remote type {remote_type}"
+        )
+    return remote_conf
+
+
 def human_time_to_int(time: str) -> Optional[int]:
     if not time:
         return None
@@ -90,9 +140,33 @@ def time_to_str(dt):
     return dt.strftime("%Y-%m-%d %H:%M:%S")
 
 
+def time_to_local(dt: Union[datetime, str]) -> datetime:
+    # TODO check usage
+    if isinstance(dt, str):
+        dt = isoparse(dt)
+    try:
+        return dt.astimezone(tz.tzlocal())
+    except (OverflowError, OSError, ValueError):
+        return dt
+
+
+def time_to_local_str(dt: Union[datetime, str]) -> str:
+    return time_to_str(time_to_local(dt))
+
+
+def is_expired(expires: Optional[Union[datetime, str]]):
+    if expires:
+        return time_to_local(expires) < time_to_local(datetime.now())
+
+    return False
+
+
+SIZE_SUFFIXES = ["", "K", "M", "G", "T", "P", "E", "Z", "Y", "R", "Q"]
+
+
 def sizeof_fmt(num, suffix="", si=False):
     power = 1000.0 if si else 1024.0
-    for unit in ["", "K", "M", "G", "T", "P", "E", "Z", "Y", "R"]:
+    for unit in SIZE_SUFFIXES[:-1]:
         if abs(num) < power:
             if not unit:
                 return f"{num:4.0f}{suffix}"
@@ -101,25 +175,7 @@ def sizeof_fmt(num, suffix="", si=False):
     return f"{num:.1f}Q{suffix}"
 
 
-def force_create_dir(name):
-    if not os.path.exists(name):
-        os.mkdir(name)
-    elif not os.path.isdir(name):
-        os.remove(name)
-        os.mkdir(name)
-
-
-def sql_str(s: str) -> str:
-    s = s.replace("'", "''")
-    return f"'{s}'"
-
-
-SIZE_SUFFIXES = ["", "K", "M", "G", "T", "P", "E", "Z", "Y", "R", "Q"]
-
-
-def suffix_to_number(num_str: str) -> Optional[int]:
-    if not num_str:
-        return None
+def suffix_to_number(num_str: str) -> int:
     try:
         if len(num_str) > 1:
             suffix = num_str[-1].upper()
@@ -131,68 +187,15 @@ def suffix_to_number(num_str: str) -> Optional[int]:
         raise ValueError(f"Invalid number/suffix for: {num_str}")
 
 
-def sql_file_filter(
-    names=None,
-    inames=None,
-    paths=None,
-    ipaths=None,
-    size=None,
-    type=None,  # pylint: disable=redefined-builtin
-    op="GLOB",
-):
-    all_filters = []
-
-    names_sql = (
-        [f"name {op} {sql_str(name)}" for name in names] if names else []
-    )
-    inames_sql = (
-        [f"LOWER(name) {op} {sql_str(iname.lower())}" for iname in inames]
-        if inames
-        else []
-    )
-    name_filter = " OR ".join(names_sql + inames_sql)
-    if name_filter:
-        all_filters.append(name_filter)
-
-    paths_sql = (
-        [f"path_str {op} {sql_str(path)}" for path in paths] if paths else []
-    )
-    ipaths_sql = (
-        [f"LOWER(path_str) {op} {sql_str(ipath.lower())}" for ipath in ipaths]
-        if ipaths
-        else []
-    )
-    path_filter = " OR ".join(paths_sql + ipaths_sql)
-    if path_filter:
-        all_filters.append(path_filter)
-
-    size_limit = suffix_to_number(size)
-    if size_limit:
-        if size_limit < 0:
-            all_filters.append(
-                f"size <= {-size_limit}"  # pylint: disable=invalid-unary-operand-type # noqa: E501
-            )
-        else:
-            all_filters.append(f"size >= {size_limit}")
-
-    type_filter = None
-    if type in ["f", "file", "files"]:
-        type_filter = "dir_type = 0"
-    elif type in ["d", "dir", "directory", "directories"]:
-        type_filter = "dir_type > 0"
-
-    if type and type_filter is None:
-        raise RuntimeError(f"Not supported 'type': {type}")
-    if type_filter:
-        all_filters.append(type_filter)
-
-    # This returns "" if no filters are specified
-    return " AND ".join(f"({f})" for f in all_filters)
+def force_create_dir(name):
+    if not os.path.exists(name):
+        os.mkdir(name)
+    elif not os.path.isdir(name):
+        os.remove(name)
+        os.mkdir(name)
 
 
-def dql_paths_join(
-    source_path: str, file_paths: Iterable[str]
-) -> Iterable[str]:
+def dql_paths_join(source_path: str, file_paths: Iterable[str]) -> Iterable[str]:
     source_parts = source_path.rstrip("/").split("/")
     if set(source_parts[-1]).intersection(GLOB_CHARS):
         # Remove last element if it is a glob match (such as *)
