@@ -1,7 +1,5 @@
 import os
 import shutil
-import sqlite3
-import uuid
 
 import pytest
 import yaml
@@ -11,42 +9,6 @@ from dql.catalog import parse_dql_file
 from dql.utils import remove_readonly
 
 
-@pytest.fixture
-def dogs_shadow_dataset(cloud_test_catalog):
-    shadow_dataset_name = uuid.uuid4().hex
-    src_uri = cloud_test_catalog.src_uri
-    catalog = cloud_test_catalog.catalog
-
-    # list source to have it in db as source for dataset
-    list(
-        catalog.ls(
-            [src_uri],
-            client_config=cloud_test_catalog.client_config,
-        )
-    )
-
-    catalog.upsert_shadow_dataset(
-        shadow_dataset_name,
-        [f"{src_uri}/dogs/*"],
-        client_config=cloud_test_catalog.client_config,
-        recursive=True,
-    )
-
-    return catalog.data_storage.get_dataset(shadow_dataset_name)
-
-
-@pytest.fixture
-def dogs_registered_dataset(cloud_test_catalog, dogs_shadow_dataset):
-    catalog = cloud_test_catalog.catalog
-    catalog.register(
-        dogs_shadow_dataset.name,
-        description="dogs dataset",
-        labels=["dogs", "dataset"],
-    )
-
-    return catalog.data_storage.get_dataset(dogs_shadow_dataset.name)
-
-
 def test_find(cloud_test_catalog):
     src_uri = cloud_test_catalog.src_uri
     catalog = cloud_test_catalog.catalog
@@ -748,6 +710,28 @@ def test_cp_dql_file_sources(cloud_test_catalog):
     assert dog_files_by_name2["others/dog4"]["size"] == 4
 
 
+@pytest.mark.parametrize("cloud_type", ["file"], indirect=True)
+def test_cp_symlinks(cloud_test_catalog):
+    catalog = cloud_test_catalog.catalog
+    src_uri = cloud_test_catalog.src_uri
+    work_dir = cloud_test_catalog.working_dir
+    dest = work_dir / "data"
+    dest.mkdir()
+    catalog.data_storage.storages.where(uri=src_uri).update(symlinks=True)
+    catalog.cp([f"{src_uri}/dogs/"], str(dest), recursive=True)
+
+    assert (dest / "dog1").is_symlink()
+    assert os.path.realpath(dest / "dog1") == str(
+        cloud_test_catalog.src / "dogs" / "dog1"
+    )
+    assert (dest / "dog1").read_text() == "woof"
+    assert (dest / "others" / "dog4").is_symlink()
+    assert os.path.realpath(dest / "others" / "dog4") == str(
+        cloud_test_catalog.src / "dogs" / "others" / "dog4"
+    )
+    assert (dest / "others" / "dog4").read_text() == "ruff"
+
+
 def test_get(cloud_test_catalog):
     src_uri = cloud_test_catalog.src_uri
     catalog = cloud_test_catalog.catalog
@@ -838,149 +822,6 @@ def test_ls_glob(cloud_test_catalog):
     ) == [("dog1", ["dog1"]), ("dog2", ["dog2"]), ("dog3", ["dog3"])]
 
 
-def test_upserting_shadow_dataset(cloud_test_catalog):
-    shadow_dataset_name = uuid.uuid4().hex
-    src_uri = cloud_test_catalog.src_uri
-    catalog = cloud_test_catalog.catalog
-
-    # list source to have it in db as source for dataset
-    list(
-        catalog.ls(
-            [src_uri],
-            client_config=cloud_test_catalog.client_config,
-        )
-    )
-
-    catalog.upsert_shadow_dataset(
-        shadow_dataset_name,
-        [f"{src_uri}/dogs/*"],
-        client_config=cloud_test_catalog.client_config,
-        recursive=True,
-    )
-
-    dataset = catalog.data_storage.get_dataset(shadow_dataset_name)
-    assert dataset.name == shadow_dataset_name
-    assert dataset.description is None
-    assert dataset.versions is None
-    assert dataset.labels == []
-    assert dataset.shadow is True
-
-    dataset_table_name = catalog.data_storage._dataset_table_name(dataset.id)
-    data = catalog.data_storage.db.execute(
-        f"select * from {dataset_table_name}"
-    ).fetchall()
-    assert data
-
-
-def test_registering_dataset(cloud_test_catalog, dogs_shadow_dataset):
-    catalog = cloud_test_catalog.catalog
-
-    catalog.register(
-        dogs_shadow_dataset.name,
-        description="dogs dataset",
-        labels=["dogs", "dataset"],
-    )
-
-    dataset = catalog.data_storage.get_dataset(dogs_shadow_dataset.name)
-    dataset_table_name = catalog.data_storage._dataset_table_name(dataset.id, 1)
-    assert dataset.name == dogs_shadow_dataset.name
-    assert dataset.description == "dogs dataset"
-    assert dataset.versions == [1]
-    assert dataset.labels == ["dogs", "dataset"]
-    assert dataset.shadow is False
-    data = catalog.data_storage.db.execute(
-        f"select * from {dataset_table_name}"
-    ).fetchall()
-    assert data
-
-
-def test_registering_dataset_with_new_name(cloud_test_catalog, dogs_shadow_dataset):
-    new_dataset_name = uuid.uuid4().hex
-    catalog = cloud_test_catalog.catalog
-
-    catalog.register(
-        dogs_shadow_dataset.name,
-        new_dataset_name=new_dataset_name,
-        description="dogs dataset",
-        labels=["dogs", "dataset"],
-    )
-    dataset = catalog.data_storage.get_dataset(new_dataset_name)
-    assert dataset
-    dataset_table_name = catalog.data_storage._dataset_table_name(dataset.id, 1)
-    assert dataset.name == new_dataset_name
-    data = catalog.data_storage.db.execute(
-        f"select * from {dataset_table_name}"
-    ).fetchall()
-    assert data
-
-
-def test_registering_dataset_with_custom_version(
-    cloud_test_catalog, dogs_shadow_dataset
-):
-    catalog = cloud_test_catalog.catalog
-
-    catalog.register(
-        dogs_shadow_dataset.name,
-        version=5,
-        description="dogs dataset",
-        labels=["dogs", "dataset"],
-    )
-
-    dataset = catalog.data_storage.get_dataset(dogs_shadow_dataset.name)
-    assert dataset.versions == [5]
-
-
-def test_removing_dataset(cloud_test_catalog, dogs_shadow_dataset):
-    catalog = cloud_test_catalog.catalog
-
-    dataset_table_name = catalog.data_storage._dataset_table_name(
-        dogs_shadow_dataset.id
-    )
-    data = catalog.data_storage.db.execute(
-        f"select * from {dataset_table_name}"
-    ).fetchall()
-    assert data
-
-    catalog.remove_dataset(dogs_shadow_dataset.name)
-    dataset = catalog.data_storage.get_dataset(dogs_shadow_dataset.name)
-    assert dataset is None
-    with pytest.raises(sqlite3.OperationalError):
-        catalog.data_storage.db.execute(
-            f"select * from {dataset_table_name}"
-        ).fetchall()
-
-
-def test_edit_dataset(cloud_test_catalog, dogs_registered_dataset):
-    dataset_new_name = uuid.uuid4().hex
-    catalog = cloud_test_catalog.catalog
-
-    catalog.edit_dataset(
-        dogs_registered_dataset.name,
-        new_name=dataset_new_name,
-        description="new description",
-        labels=["cats", "birds"],
-    )
-
-    dataset = catalog.data_storage.get_dataset(dataset_new_name)
-    assert dataset.versions == [1]
-    assert dataset.name == dataset_new_name
-    assert dataset.description == "new description"
-    assert dataset.labels == ["cats", "birds"]
-
-
-def test_ls_dataset_rows(cloud_test_catalog, dogs_registered_dataset):
-    catalog = cloud_test_catalog.catalog
-
-    assert {
-        r.name for r in catalog.ls_dataset_rows(dogs_registered_dataset.name, version=1)
-    } == {
-        "dog1",
-        "dog2",
-        "dog3",
-        "dog4",
-    }
-
-
 def clear_storages(catalog):
     catalog.data_storage.storages.delete()
 
