@@ -9,12 +9,12 @@ import pytest
 from dql.cli import index, ls
 
 
+def same_lines(lines1, lines2):
+    return sorted(lines1.split("\n")) == sorted(lines2.split("\n"))
+
+
 def test_ls_no_args(cloud_test_catalog, capsys):
-    # These tests re-use the same database for better performance
-    conn = cloud_test_catalog.catalog.data_storage.db
-    conn.execute("DELETE FROM buckets")
-    conn.commit()
-    src = str(cloud_test_catalog.src).rstrip("/")
+    src = cloud_test_catalog.src_uri
     catalog = cloud_test_catalog.catalog
     conf = cloud_test_catalog.client_config
     index([src], catalog=catalog, client_config=conf)
@@ -27,8 +27,8 @@ def test_ls_no_args(cloud_test_catalog, capsys):
     assert captured.out == f"{src}\n"
 
 
-def test_ls_root(cloud_test_catalog, capsys):
-    src = str(cloud_test_catalog.src).rstrip("/")
+def test_ls_root(cloud_test_catalog, cloud_type, capsys):
+    src = cloud_test_catalog.src_uri
     root = src[: src.find("://") + 3]
     src_name = src.replace(root, "", 1)
     ls(
@@ -37,8 +37,11 @@ def test_ls_root(cloud_test_catalog, capsys):
         client_config=cloud_test_catalog.client_config,
     )
     captured = capsys.readouterr()
-    buckets = captured.out.split("\n")
-    assert src_name in buckets
+    if cloud_type == "file":
+        assert not captured.out
+    else:
+        buckets = captured.out.split("\n")
+        assert src_name in buckets
 
 
 ls_sources_output = """\
@@ -57,18 +60,18 @@ dog3
 
 
 def test_ls_sources(cloud_test_catalog, capsys):
-    src = str(cloud_test_catalog.src).rstrip("/")
+    src = cloud_test_catalog.src_uri
     ls(
         [src, f"{src}/dogs/*"],
         catalog=cloud_test_catalog.catalog,
         client_config=cloud_test_catalog.client_config,
     )
     captured = capsys.readouterr()
-    assert captured.out == ls_sources_output
+    assert same_lines(captured.out, ls_sources_output)
 
 
 def test_ls_not_found(cloud_test_catalog):
-    src = str(cloud_test_catalog.src).rstrip("/")
+    src = cloud_test_catalog.src_uri
     with pytest.raises(FileNotFoundError):
         ls(
             [src, f"{src}/cats/bogus*"],
@@ -78,10 +81,8 @@ def test_ls_not_found(cloud_test_catalog):
 
 
 def test_ls_partial_indexing(cloud_test_catalog, capsys):
-    # These tests re-use the same database for better performance
     data_storage = cloud_test_catalog.catalog.data_storage
-    data_storage.storages.delete()
-    src = str(cloud_test_catalog.src).rstrip("/")
+    src = cloud_test_catalog.src_uri
     src_storage = data_storage.clone(src)
 
     ls(
@@ -95,7 +96,7 @@ def test_ls_partial_indexing(cloud_test_catalog, capsys):
     captured = capsys.readouterr()
     assert src_storage.partials.col("path_str") == ["dogs/others/"]
     assert "Listing" in captured.err
-    assert "dog4" in captured.out
+    assert "dog4\n" == captured.out
 
     ls(
         [f"{src}/cats/"],
@@ -106,7 +107,7 @@ def test_ls_partial_indexing(cloud_test_catalog, capsys):
     captured = capsys.readouterr()
     assert src_storage.partials.col("path_str") == ["cats/", "dogs/others/"]
     assert "Listing" in captured.err
-    assert "cat1\ncat2" in captured.out
+    assert same_lines("cat1\ncat2\n", captured.out)
 
     ls(
         [f"{src}/dogs/"],
@@ -117,7 +118,7 @@ def test_ls_partial_indexing(cloud_test_catalog, capsys):
     captured = capsys.readouterr()
     assert src_storage.partials.col("path_str") == ["cats/", "dogs/"]
     assert "Listing" in captured.err
-    assert "others/\ndog1\ndog2\ndog3" in captured.out
+    assert same_lines("others/\ndog1\ndog2\ndog3\n", captured.out)
 
     ls(
         [f"{src}/cats/"],
@@ -128,7 +129,7 @@ def test_ls_partial_indexing(cloud_test_catalog, capsys):
     captured = capsys.readouterr()
     assert src_storage.partials.col("path_str") == ["cats/", "dogs/"]
     assert "Listing" not in captured.err
-    assert "cat1\ncat2" in captured.out
+    assert same_lines("cat1\ncat2\n", captured.out)
 
     ls(
         [f"{src}/"],
@@ -139,7 +140,7 @@ def test_ls_partial_indexing(cloud_test_catalog, capsys):
     captured = capsys.readouterr()
     assert src_storage.partials.col("path_str") == []
     assert "Listing" in captured.err
-    assert "cats/\ndogs/\ndescription" in captured.out
+    assert same_lines("cats/\ndogs/\ndescription\n", captured.out)
 
 
 class MockResponse:
